@@ -7,7 +7,6 @@ import datetime
 import codecs
 import binascii
 import collections
-import struct
 from test import support
 from io import BytesIO
 
@@ -361,6 +360,13 @@ class TestPlistlib(unittest.TestCase):
                                   plistlib.dumps,
                                   testString)
 
+    def test_non_bmp_characters(self):
+        pl = {'python': '\U0001f40d'}
+        for fmt in ALL_FORMATS:
+            with self.subTest(fmt=fmt):
+                data = plistlib.dumps(pl, fmt=fmt)
+                self.assertEqual(plistlib.loads(data), pl)
+
     def test_nondictroot(self):
         for fmt in ALL_FORMATS:
             with self.subTest(fmt=fmt):
@@ -527,8 +533,14 @@ class TestPlistlibDeprecated(unittest.TestCase):
         self.assertEqual(cur, in_data)
 
 
+class MiscTestCase(unittest.TestCase):
+    def test__all__(self):
+        blacklist = {"PlistFormat", "PLISTHEADER"}
+        support.check__all__(self, plistlib, blacklist=blacklist)
+
+
 def test_main():
-    support.run_unittest(TestPlistlib, TestPlistlibDeprecated)
+    support.run_unittest(TestPlistlib, TestPlistlibDeprecated, MiscTestCase)
 
 
 if __name__ == '__main__':
